/* ==============================================================================
 * Author:  Jonathan Weaver, jonw0224@aim.com
 * E-mail Contact: jonw0224@aim.com
 * Description:  Serial interface
 * Version: 3.1
 * Date: 4/25/2014
 * Filename:  serial.h
 *
 * Versions History:
 *                    6/3/2010 -  Created file
 *                    3/8/2013 -  Added setPortRTS, setPortDTR
 *                   1/10/2014 -  Added support for multiple COM ports
 *                   3/13/2014 -  Added serialBytesAvailable.
 *                   4/25/2014 -  Fixed unsigned character bug.
 *
 * Copyright (C) 2010-2014 Jonathan Weaver
 *
 * This file is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ==============================================================================
 */

/* Closes the serial port
 * hComm: the handle to the Comm Port
 */
int serialPortClose(int hComm);

/* Initializes and opens the serial port
 * portName: the name of the comm port to open (e.g. "COM1")
 * configString: configuration string for the port including baud rate, etc. (e.g. "57600,n,8,1")
 * returns:  1 if successful, -1 if port failed to open, -2 if failed to build DCM, -3 if failed to apply DCM to serial port, -4 if failed to set timeouts.
 */
int serialPortOpen(char portName[], char configStr[]);

/* Sets the port timeouts.
 * hComm: the handle to the Comm Port
 * rdInterval: the timeout for each byte read in milliseconds
 * rdTotalMult: the multiplier for each byte read
 * rdTotalCosnt: an additional timeout to allow at the end of a recieve in milliseconds
 * wrTotalMult: the multiplier for each byte write
 * wrTotalConst: the additional timeout to allow at the end of transmit in milliseconds
 * returns: 1 if successful, 0 if unsuccessful */
int setPortTimeouts(int hComm, int rdInterval, int rdTotalMult, int rdTotalConst, int wrTotalMult, int wrTotalConst);

/* Transmits a string over the serial port
 * hComm: the handle to the Comm Port
 * strToWrite: the string to transmit
 * noOfBytesToWrite: the number of bytes of the string to transmit
 * returns: 0 if failed, number of bytes written if successful
 */
int serialTXString(int hComm, unsigned char strToWrite[], int noOfBytesToWrite);

/* Transmits a character over the serial port
 * hComm: the handle to the Comm Port
 * ch: the character to transmit
 * returns: 0 if failed, number of bytes written if successful
 */
int serialTXChar(int hComm, unsigned char ch);

/* Recieves a string over the serial port
 * hComm: the handle to the Comm Port
 * strToRead: a buffer for the string to recieve
 * noOfBytesToRead: the number of bytes of the string to recieve
 * returns: 0 if failed, number of bytes read if successful
 */
int serialRXString(int hComm, unsigned char strToRead[], int noOfBytesToRead);

/* Recieves a character over the serial port
 * hComm: the handle to the Comm Port
 * ch: a character to read
 * returns: 0 if failed, number of bytes read if successful
 */
int serialRXChar(int hComm, unsigned char* ch);

/* Sets the RTS signal on the comm port
 * hComm: the handle to the Comm Port
 * rts: 1 to set RTS, 0 to clear RTS
 * returns: nonzero if successful, 0 if unsuccessful */
int setPortRTS(int hComm, int rts);

/* Sets the DTR signal on the comm port
 * hComm: the handle to the Comm Port
 * dtr: 1 to set DTR, 0 to clear DTR
 * returns: nonzero is successful, 0 if unsuccessful */
int setPortDTR(int hComm, int dtr);

/* Returns the bytes available in the recieve buffer to be read
 * hComm: then handle to the Comm poet
 * returns: the number of bytes available in the recieve buffer to be read */
int serialPortBytesAvailable(int hComm);

/* Sets the read and write buffersizes to be used with the serial port
 * hComm: the handle to the Comm Port
 * readSize: the size of the read buffer
 * writeSize: the size of the write buffer */
int serialSetBufferSize(int hComm, unsigned int readSize, unsigned int writeSize);

