% ==============================================================================
% Author:  Jonathan Weaver, jonw0224@aim.com
% E-mail Contact: jonw0224@aim.com
% Description:  Serial interface 
% Version: 3.2
% Date: 5/2/2014
% Filename:  serialload.m
%
% Versions History:
%                    2/13/2012 -  Created file
%                    8/29/2012 -  Updated serial.c, serial.h, and added
%                                 dllimport of setPortTimeouts.
%					 2/11/2013 -  Fixed serialRXChar.  Removed functions
%                                 serialRXString and serialTXString.
%					 3/08/2013 -  Added setPortRTS, setPortDTR functions.
%					 1/10/2014 -  Added support for multiple serial ports.
%                    3/13/2014 -  Added serialTXString and serialRXString.
%                                 Added serialPortBytesAvailable.
%                    4/15/2014 -  Automatically loads 32 bit or 64 bit DLL,
%                                 whichever is appropriate.  All calls to
%                                 the DLL have m file wrappers.
%                    4/25/2014 -  Fixed unsigned bug in DLL.
%					 5/02/2014 -  Fixed import call for serialTXString
%
% Copyright (C) 2012-2014 Jonathan Weaver
%
% This file is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% ==============================================================================


try
    % Closes the serial port 
    % hComm: the handle to the serial port
    import('serial64.dll', 'sPortClose', 'serialPortClose_internal', 'int16', 'int32 hComm');
    
    % Initializes and opens the serial port
    % portName: the name of the comm port to open (e.g. "COM1")
    % configString: configuration string for the port including baud rate, etc. (e.g. "57600,n,8,1")
    % returns:  handle to the serial port if successful, -1 if port failed to open, -2 if failed to build DCM, -3 if failed to apply DCM to serial port, -4 if failed to set timeouts.
    import('serial64.dll', 'sPortOpen', 'serialPortOpen_internal', 'int16', 'string portName, string configStr');
    
    % Sets the port timeouts.
    % hComm: the handle to the serial port
    % rdInterval: the timeout for each byte read in milliseconds
    % rdTotalMult: the multiplier for each byte read
    % rdTotalCosnt: an additional timeout to allow at the end of a recieve in milliseconds
    % wrTotalMult: the multiplier for each byte write
    % wrTotalConst: the additional timeout to allow at the end of transmit in milliseconds
    % returns: 1 if successful, 0 if unsuccessful
    import('serial64.dll', 'sPortTimeouts', 'setPortTimeouts_internal', 'int16', 'int32 hComm, int16 rdInterval, int16 rdTotalMult, int16 rdTotalConst, int16 wrTotalMult, int16 wrTotalConst');
    
    % Transmits a character over the serial port
    % hComm: the handle to the serial port
    % ch: the character to transmit
    % returns: 0 if failed, 1 if successful
    import('serial64.dll', 'sTXChar', 'serialTXChar_internal', 'int16', 'int32 hComm, uint8 ch');
    
    % Recieves a character over the serial port
    % hComm: the handle to the serial port
    % returns: -1 if failed, the character read if successful
    import('serial64.dll', 'sRXChar', 'serialRXChar_internal', 'int16', 'int32 hComm');
    
    % Sets the RTS signal on the comm port
    % hComm: the handle to the serial port
    % rts: 1 to set RTS, 0 to clear RTS
    % returns: nonzero if successful, 0 if unsuccessful */
    import('serial64.dll', 'sPortRTS', 'setPortRTS_internal', 'int16', 'int32 hComm, int16 rts');
    
    % Sets the DTR signal on the comm port
    % hComm: the handle to the serial port
    % dtr: 1 to set DTR, 0 to clear DTR
    % returns: nonzero is successful, 0 if unsuccessful */
    import('serial64.dll', 'sPortDTR', 'setPortDTR_internal', 'int16', 'int32 hComm, int16 dtr');
    
    % Returns the bytes available in the recieve buffer to be read
    % hComm: the handle to the comm port
    % returns: the number of bytes available in the recieve buffer to be read */
    import('serial64.dll', 'sPortBytesAvailable', 'serialPortBytesAvailable_internal', 'int16', 'int32 hComm');
    
    % Recieves a string over the serial port
    % hComm: the handle to the Comm Port
    % strToRead: a buffer for the string to recieve
    % noOfBytesToRead: the number of bytes of the string to recieve
    % returns: 0 if failed, number of bytes read if successful
    import('serial64.dll', 'sRXString', 'serialRXString_internal', 'int16', 'int32 hComm, uint8 &strToRead, int32 noOfBytesToRead');
    
    % Transmits a string over the serial port
    % hComm: the handle to the Comm Port
    % strToWrite: the string to transmit
    % noOfBytesToWrite: the number of bytes of the string to transmit
    % returns: 0 if failed, number of bytes written if successful
    import('serial64.dll', 'sTXString', 'serialTXString_internal', 'int16', 'int32 hComm, uint8 &strToWrite, int32 noOfBytesToWrite');

    % Sets the read and write buffersizes to be used with the serial port
    % hComm: the handle to the Comm Port
    % readSize: the size of the read buffer
    % writeSize: the size of the write buffer */
    import('serial64.dll', 'sSetBufferSize', 'serialSetBufferSize_internal', 'int16', 'int32 hComm, uint32 readSize, uint32 writeSize');

catch

    % Closes the serial port 
    % hComm: the handle to the serial port
    import('serial32.dll', 'sPortClose', 'serialPortClose_internal', 'int16', 'int32 hComm');
    
    % Initializes and opens the serial port
    % portName: the name of the comm port to open (e.g. "COM1")
    % configString: configuration string for the port including baud rate, etc. (e.g. "57600,n,8,1")
    % returns:  handle to the serial port if successful, -1 if port failed to open, -2 if failed to build DCM, -3 if failed to apply DCM to serial port, -4 if failed to set timeouts.
    import('serial32.dll', 'sPortOpen', 'serialPortOpen_internal', 'int16', 'string portName, string configStr');
    
    % Sets the port timeouts.
    % hComm: the handle to the serial port
    % rdInterval: the timeout for each byte read in milliseconds
    % rdTotalMult: the multiplier for each byte read
    % rdTotalCosnt: an additional timeout to allow at the end of a recieve in milliseconds
    % wrTotalMult: the multiplier for each byte write
    % wrTotalConst: the additional timeout to allow at the end of transmit in milliseconds
    % returns: 1 if successful, 0 if unsuccessful
    import('serial32.dll', 'sPortTimeouts', 'setPortTimeouts_internal', 'int16', 'int32 hComm, int16 rdInterval, int16 rdTotalMult, int16 rdTotalConst, int16 wrTotalMult, int16 wrTotalConst');
    
    % Transmits a character over the serial port
    % hComm: the handle to the serial port
    % ch: the character to transmit
    % returns: 0 if failed, 1 if successful
    import('serial32.dll', 'sTXChar', 'serialTXChar_internal', 'int16', 'int32 hComm, uint8 ch');
    
    % Recieves a character over the serial port
    % hComm: the handle to the serial port
    % returns: -1 if failed, the character read if successful
    import('serial32.dll', 'sRXChar', 'serialRXChar_internal', 'int16', 'int32 hComm');
    
    % Sets the RTS signal on the comm port
    % hComm: the handle to the serial port
    % rts: 1 to set RTS, 0 to clear RTS
    % returns: nonzero if successful, 0 if unsuccessful */
    import('serial32.dll', 'sPortRTS', 'setPortRTS_internal', 'int16', 'int32 hComm, int16 rts');
    
    % Sets the DTR signal on the comm port
    % hComm: the handle to the serial port
    % dtr: 1 to set DTR, 0 to clear DTR
    % returns: nonzero is successful, 0 if unsuccessful */
    import('serial32.dll', 'sPortDTR', 'setPortDTR_internal', 'int16', 'int32 hComm, int16 dtr');
    
    % Returns the bytes available in the recieve buffer to be read
    % hComm: then handle to the Comm poet
    % returns: the number of bytes available in the recieve buffer to be read */
    import('serial32.dll', 'sPortBytesAvailable', 'serialPortBytesAvailable_internal', 'int16', 'int32 hComm');
    
    % Recieves a string over the serial port
    % hComm: the handle to the Comm Port
    % strToRead: a buffer for the string to recieve
    % noOfBytesToRead: the number of bytes of the string to recieve
    % returns: 0 if failed, number of bytes read if successful
    import('serial32.dll', 'sRXString', 'serialRXString_internal', 'int16', 'int32 hComm, uint8 &strToRead, int32 noOfBytesToRead');
    
    % Transmits a string over the serial port
    % hComm: the handle to the Comm Port
    % strToWrite: the string to transmit
    % noOfBytesToWrite: the number of bytes of the string to transmit
    % returns: 0 if failed, number of bytes written if successful
    import('serial32.dll', 'sTXString', 'serialTXString_internal', 'int16', 'int32 hComm, uint8 &strToWrite, int32 noOfBytesToWrite');
    
    % Sets the read and write buffersizes to be used with the serial port
    % hComm: the handle to the Comm Port
    % readSize: the size of the read buffer
    % writeSize: the size of the write buffer
    import('serial32.dll', 'sSetBufferSize', 'serialSetBufferSize_internal', 'int16', 'int32 hComm, int32 readSize, int32 writeSize');

end


